@@ -46,6 +46,14 @@
 #define D6i      0x1A
 #define D9tx     0x1B
 #define DX       0x1C
+#define VYPERNOVO 0x1D
+#define ZOOPNOVO  0x1E
+
+#define ID_D6I_V1_MIX2 0x1871C062
+#define ID_D6I_V1_MIX3 0x1871C063
+#define ID_D6I_V2      0x18724062
+#define ID_D4I_V1      ID_D6I_V1_MIX2
+#define ID_D4I_V2      ID_D6I_V2
 
 #define AIR      0
 #define NITROX   1
@@ -120,6 +128,9 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		return DC_STATUS_SUCCESS;
 	}
 
+	// Get the logbook id tag.
+	unsigned int id = array_uint32_le (data + 1);
+
 	// Gasmix information.
 	unsigned int gasmode_offset = 0x19;
 	unsigned int gasmix_offset = 0x21;
@@ -128,14 +139,20 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		gasmode_offset = 0x1F;
 		gasmix_offset = 0x54;
 		gasmix_count = 8;
-	} else if (parser->model == D4i) {
+	} else if (parser->model == D4i || parser->model == ZOOPNOVO) {
 		gasmode_offset = 0x1D;
-		gasmix_offset = 0x5F;
+		if (id == ID_D4I_V2)
+			gasmix_offset = 0x67;
+		else
+			gasmix_offset = 0x5F;
 		gasmix_count = 1;
-	} else if (parser->model == D6i) {
+	} else if (parser->model == D6i || parser->model == VYPERNOVO) {
 		gasmode_offset = 0x1D;
-		gasmix_offset = 0x5F;
-		if (data[1] == 0x63)
+		if (id == ID_D6I_V2)
+			gasmix_offset = 0x67;
+		else
+			gasmix_offset = 0x5F;
+		if (id == ID_D6I_V1_MIX3 || id == ID_D6I_V2)
 			gasmix_count = 3;
 		else
 			gasmix_count = 2;
@@ -155,7 +172,8 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		config += 1;
 	} else if (parser->model == HELO2 || parser->model == D4i ||
 		parser->model == D6i || parser->model == D9tx ||
-		parser->model == DX) {
+		parser->model == DX || parser->model == ZOOPNOVO ||
+		parser->model == VYPERNOVO) {
 		config = gasmix_offset + gasmix_count * 6;
 	}
 	if (config + 1 > size)
@@ -175,7 +193,8 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		for (unsigned int i = 0; i < gasmix_count; ++i) {
 			if (parser->model == HELO2 || parser->model == D4i ||
 				parser->model == D6i || parser->model == D9tx ||
-				parser->model == DX) {
+				parser->model == DX || parser->model == ZOOPNOVO ||
+				parser->model == VYPERNOVO) {
 				parser->oxygen[i] = data[gasmix_offset + 6 * i + 1];
 				parser->helium[i] = data[gasmix_offset + 6 * i + 2];
 			} else {
@@ -192,8 +211,13 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		if (parser->model == HELO2) {
 			parser->gasmix = data[0x26];
 		} else if (parser->model == D4i || parser->model == D6i ||
-			parser->model == D9tx) {
-			parser->gasmix = data[0x28];
+			parser->model == D9tx || parser->model == ZOOPNOVO ||
+			parser->model == VYPERNOVO) {
+			if (id == ID_D4I_V2 || id == ID_D6I_V2) {
+				parser->gasmix = data[0x2D];
+			} else {
+				parser->gasmix = data[0x28];
+			}
 		}
 	}
 	parser->config = config;
@@ -264,7 +288,9 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 	unsigned int offset = 0x11;
 	if (parser->model == HELO2 || parser->model == DX)
 		offset = 0x17;
-	else if (parser->model == D4i || parser->model == D6i || parser->model == D9tx)
+	else if (parser->model == D4i || parser->model == D6i ||
+		parser->model == D9tx || parser->model == ZOOPNOVO ||
+		parser->model == VYPERNOVO)
 		offset = 0x13;
 
 	if (abstract->size < offset + 7)
@@ -274,7 +300,8 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 
 	if (datetime) {
 		if (parser->model == D4i || parser->model == D6i ||
-			parser->model == D9tx || parser->model == DX) {
+			parser->model == D9tx || parser->model == DX ||
+			parser->model == ZOOPNOVO || parser->model == VYPERNOVO) {
 			datetime->year   = p[0] + (p[1] << 8);
 			datetime->month  = p[2];
 			datetime->day    = p[3];
@@ -320,7 +347,8 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 			if (parser->model == D4)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0B);
 			else if (parser->model == D4i || parser->model == D6i ||
-				parser->model == D9tx || parser->model == DX)
+				parser->model == D9tx || parser->model == DX ||
+				parser->model == ZOOPNOVO || parser->model == VYPERNOVO)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D);
 			else if (parser->model == HELO2)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D) * 60;
@@ -441,7 +469,8 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 	// Sample recording interval.
 	unsigned int interval_sample_offset = 0x18;
 	if (parser->model == HELO2 || parser->model == D4i ||
-		parser->model == D6i || parser->model == D9tx)
+		parser->model == D6i || parser->model == D9tx ||
+		parser->model == ZOOPNOVO || parser->model == VYPERNOVO)
 		interval_sample_offset = 0x1E;
 	else if (parser->model == DX)
 		interval_sample_offset = 0x22;
